@@ -41,11 +41,14 @@
 
 #include "gs-debug.h"
 
+#define MAX_FAILURES 5
+
 static gboolean verbose        = FALSE;
 static gboolean show_version   = FALSE;
 static gboolean enable_logout  = FALSE;
 static gboolean enable_switch  = FALSE;
 static char    *logout_command = NULL;
+static char    *away_message   = NULL;
 
 static GOptionEntry entries [] = {
         { "verbose", 0, 0, G_OPTION_ARG_NONE, &verbose,
@@ -58,6 +61,8 @@ static GOptionEntry entries [] = {
           N_("Command to invoke from the logout button"), NULL },
         { "enable-switch", 0, 0, G_OPTION_ARG_NONE, &enable_switch,
           N_("Show the switch user button"), NULL },
+        { "away-message", 0, 0, G_OPTION_ARG_STRING, &away_message,
+          N_("Message to show in the dialog"), N_("MESSAGE") },
         { NULL }
 };
 
@@ -138,7 +143,7 @@ request_response (GSLockPlug *plug,
         char *text;
 
         gs_lock_plug_set_sensitive (plug, TRUE);
-        gs_lock_plug_show_prompt (plug, prompt, visible);
+        gs_lock_plug_enable_prompt (plug, prompt, visible);
         response = gs_lock_plug_run (plug);
 
         gs_debug ("got response: %d", response);
@@ -147,6 +152,7 @@ request_response (GSLockPlug *plug,
         if (response == GS_LOCK_PLUG_RESPONSE_OK) {
                 gs_lock_plug_get_text (plug, &text);
         }
+        gs_lock_plug_disable_prompt (plug);
 
         return text;
 }
@@ -155,47 +161,47 @@ request_response (GSLockPlug *plug,
 static const char *
 maybe_translate_message (const char *msg)
 {
-	char              *s;
-	const char        *ret;
-	static GHashTable *hash = NULL;
+        char              *s;
+        const char        *ret;
+        static GHashTable *hash = NULL;
 
-	if (hash == NULL) {
-		/* Here we come with some fairly standard messages so that
-		   we have as much as possible translated.  Should really be
-		   translated in pam I suppose.  This way we can "change"
-		   some of these messages to be more sane. */
-		hash = g_hash_table_new (g_str_hash, g_str_equal);
-		/* login: is whacked always translate to Username: */
-		g_hash_table_insert (hash, "login:", _("Username:"));
-		g_hash_table_insert (hash, "Username:", _("Username:"));
-		g_hash_table_insert (hash, "username:", _("Username:"));
-		g_hash_table_insert (hash, "Password:", _("Password:"));
-		g_hash_table_insert (hash, "password:", _("Password:"));
-		g_hash_table_insert (hash, "You are required to change your password immediately (password aged)", _("You are required to change your password immediately (password aged)"));
-		g_hash_table_insert (hash, "You are required to change your password immediately (root enforced)", _("You are required to change your password immediately (root enforced)"));
-		g_hash_table_insert (hash, "Your account has expired; please contact your system administrator", _("Your account has expired; please contact your system administrator"));
-		g_hash_table_insert (hash, "No password supplied", _("No password supplied"));
-		g_hash_table_insert (hash, "Password unchanged", _("Password unchanged"));
-		g_hash_table_insert (hash, "Can not get username", _("Can not get username"));
-		g_hash_table_insert (hash, "Retype new UNIX password:", _("Retype new UNIX password:"));
-		g_hash_table_insert (hash, "Enter new UNIX password:", _("Enter new UNIX password:"));
-		g_hash_table_insert (hash, "(current) UNIX password:", _("(current) UNIX password:"));
-		g_hash_table_insert (hash, "Error while changing NIS password.", _("Error while changing NIS password."));
-		g_hash_table_insert (hash, "You must choose a longer password", _("You must choose a longer password"));
-		g_hash_table_insert (hash, "Password has been already used. Choose another.", _("Password has been already used. Choose another."));
-		g_hash_table_insert (hash, "You must wait longer to change your password", _("You must wait longer to change your password"));
-		g_hash_table_insert (hash, "Sorry, passwords do not match", _("Sorry, passwords do not match"));
-		/* FIXME: what about messages which have some variables in them, perhaps try to do those as well */
-	}
+        if (hash == NULL) {
+                /* Here we come with some fairly standard messages so that
+                   we have as much as possible translated.  Should really be
+                   translated in pam I suppose.  This way we can "change"
+                   some of these messages to be more sane. */
+                hash = g_hash_table_new (g_str_hash, g_str_equal);
+                /* login: is whacked always translate to Username: */
+                g_hash_table_insert (hash, "login:", _("Username:"));
+                g_hash_table_insert (hash, "Username:", _("Username:"));
+                g_hash_table_insert (hash, "username:", _("Username:"));
+                g_hash_table_insert (hash, "Password:", _("Password:"));
+                g_hash_table_insert (hash, "password:", _("Password:"));
+                g_hash_table_insert (hash, "You are required to change your password immediately (password aged)", _("You are required to change your password immediately (password aged)"));
+                g_hash_table_insert (hash, "You are required to change your password immediately (root enforced)", _("You are required to change your password immediately (root enforced)"));
+                g_hash_table_insert (hash, "Your account has expired; please contact your system administrator", _("Your account has expired; please contact your system administrator"));
+                g_hash_table_insert (hash, "No password supplied", _("No password supplied"));
+                g_hash_table_insert (hash, "Password unchanged", _("Password unchanged"));
+                g_hash_table_insert (hash, "Can not get username", _("Can not get username"));
+                g_hash_table_insert (hash, "Retype new UNIX password:", _("Retype new UNIX password:"));
+                g_hash_table_insert (hash, "Enter new UNIX password:", _("Enter new UNIX password:"));
+                g_hash_table_insert (hash, "(current) UNIX password:", _("(current) UNIX password:"));
+                g_hash_table_insert (hash, "Error while changing NIS password.", _("Error while changing NIS password."));
+                g_hash_table_insert (hash, "You must choose a longer password", _("You must choose a longer password"));
+                g_hash_table_insert (hash, "Password has been already used. Choose another.", _("Password has been already used. Choose another."));
+                g_hash_table_insert (hash, "You must wait longer to change your password", _("You must wait longer to change your password"));
+                g_hash_table_insert (hash, "Sorry, passwords do not match", _("Sorry, passwords do not match"));
+                /* FIXME: what about messages which have some variables in them, perhaps try to do those as well */
+        }
 
-	s = g_strstrip (g_strdup (msg));
-	ret = g_hash_table_lookup (hash, s);
-	g_free (s);
+        s = g_strstrip (g_strdup (msg));
+        ret = g_hash_table_lookup (hash, s);
+        g_free (s);
 
-	if (ret != NULL) {
-		return ret;
+        if (ret != NULL) {
+                return ret;
         } else {
-		return msg;
+                return msg;
         }
 }
 
@@ -214,6 +220,9 @@ auth_message_handler (GSAuthMessageStyle style,
         gs_profile_start (NULL);
         gs_debug ("Got message style %d: '%s'", style, msg);
 
+        gtk_widget_show (GTK_WIDGET (plug));
+        gs_lock_plug_set_ready (plug);
+
         ret = TRUE;
         *response = NULL;
         message = maybe_translate_message (msg);
@@ -256,6 +265,7 @@ auth_message_handler (GSAuthMessageStyle style,
                 gtk_main_iteration ();
         }
 
+        gs_lock_plug_set_busy (plug);
         gs_profile_end (NULL);
 
         return ret;
@@ -278,6 +288,8 @@ do_auth_check (GSLockPlug *plug)
 
         error = NULL;
 
+        gs_lock_plug_disable_prompt (plug);
+        gs_lock_plug_set_busy (plug);
         res = gs_auth_verify_user (g_get_user_name (), g_getenv ("DISPLAY"), auth_message_handler, plug, &error);
 
         gs_debug ("Verify user returned: %s", res ? "TRUE" : "FALSE");
@@ -289,8 +301,6 @@ do_auth_check (GSLockPlug *plug)
                         gs_lock_plug_show_message (plug, _("Authentication failed."));
                 }
 
-                g_timeout_add (3000, (GSourceFunc)reset_idle_cb, plug);
-
                 printf ("NOTICE=AUTH FAILED\n");
                 fflush (stdout);
 
@@ -306,7 +316,8 @@ static void
 response_cb (GSLockPlug *plug,
              gint        response_id)
 {
-        if (response_id == GS_LOCK_PLUG_RESPONSE_CANCEL) {
+        if ((response_id == GS_LOCK_PLUG_RESPONSE_CANCEL) ||
+            (response_id == GTK_RESPONSE_DELETE_EVENT)) {
                 quit_response_cancel ();
         }
 }
@@ -314,15 +325,35 @@ response_cb (GSLockPlug *plug,
 static gboolean
 auth_check_idle (GSLockPlug *plug)
 {
-        gboolean res;
+        gboolean     res;
+        gboolean     again;
+        static guint loop_counter = 0;
 
+        again = TRUE;
         res = do_auth_check (plug);
 
         if (res) {
+                again = FALSE;
                 g_idle_add ((GSourceFunc)quit_response_ok, NULL);
+        } else {
+                loop_counter++;
+
+                if (loop_counter < MAX_FAILURES) {
+                        g_timeout_add (3000, (GSourceFunc)reset_idle_cb, plug);
+                } else {
+                        again = FALSE;
+                        gtk_main_quit ();
+                }
         }
 
-        return !res;
+        return again;
+}
+
+static void
+show_cb (GtkWidget *widget,
+         gpointer   data)
+{
+        print_id (widget);
 }
 
 static gboolean
@@ -346,11 +377,14 @@ popup_dialog_idle (void)
                 g_object_set (widget, "switch-enabled", TRUE, NULL);
         }
 
+        if (away_message) {
+                g_object_set (widget, "away-message", away_message, NULL);
+        }
+
         g_signal_connect (GS_LOCK_PLUG (widget), "response", G_CALLBACK (response_cb), NULL);
+        g_signal_connect (widget, "show", G_CALLBACK (show_cb), NULL);
 
-        gtk_widget_show (widget);
-
-        print_id (widget);
+        gtk_widget_realize (widget);
 
         g_idle_add ((GSourceFunc)auth_check_idle, widget);
 
@@ -390,7 +424,7 @@ privileged_initialization (int     *argc,
                         &uid_message);
 
         if (nolock_reason) {
-                g_warning ("Locking disabled: %s", nolock_reason);
+                g_debug ("Locking disabled: %s", nolock_reason);
         }
 
         if (uid_message && verbose) {
@@ -419,14 +453,12 @@ lock_initialization (int     *argc,
                      char   **nolock_reason,
                      gboolean verbose)
 {
-        gs_profile_start (NULL);
-
-        if (nolock_reason) {
+        if (nolock_reason != NULL) {
                 *nolock_reason = NULL;
         }
 
 #ifdef NO_LOCKING
-        if (nolock_reason) {
+        if (nolock_reason != NULL) {
                 *nolock_reason = g_strdup ("not compiled with locking support");
         }
 
@@ -435,7 +467,7 @@ lock_initialization (int     *argc,
 
         /* Finish initializing locking, now that we're out of privileged code. */
         if (! gs_auth_init ()) {
-                if (nolock_reason) {
+                if (nolock_reason != NULL) {
                         *nolock_reason = g_strdup ("error getting password");
                 }
 
@@ -447,7 +479,7 @@ lock_initialization (int     *argc,
            locking just in case.
         */
         if (getenv ("RUNNING_UNDER_GDM")) {
-                if (nolock_reason) {
+                if (nolock_reason != NULL) {
                         *nolock_reason = g_strdup ("running under GDM");
                 }
 
@@ -471,7 +503,7 @@ lock_initialization (int     *argc,
 #endif
 
                 if (macos) {
-                        if (nolock_reason) {
+                        if (nolock_reason != NULL) {
                                 *nolock_reason = g_strdup ("Cannot lock securely on MacOS X");
                         }
 
@@ -481,8 +513,6 @@ lock_initialization (int     *argc,
 
 #endif /* NO_LOCKING */
 
-        gs_profile_end (NULL);
-
         return TRUE;
 }
 
@@ -501,14 +531,12 @@ main (int    argc,
         textdomain (GETTEXT_PACKAGE);
 #endif
 
-        g_type_init ();
-
-        if (error) {
-                fprintf (stderr, "%s\n", error->message);
-                response_lock_init_failed ();
-                exit (1);
+        if (! g_thread_supported ()) {
+                g_thread_init (NULL);
         }
 
+        g_type_init ();
+
         gs_profile_start (NULL);
 
         if (! privileged_initialization (&argc, argv, verbose)) {
@@ -531,8 +559,8 @@ main (int    argc,
         }
 
         if (! lock_initialization (&argc, argv, &nolock_reason, verbose)) {
-                if (nolock_reason) {
-                        g_warning ("Screen locking disabled: %s", nolock_reason);
+                if (nolock_reason != NULL) {
+                        g_debug ("Screen locking disabled: %s", nolock_reason);
                         g_free (nolock_reason);
                 }
                 response_lock_init_failed ();
@@ -548,5 +576,5 @@ main (int    argc,
         gs_profile_end (NULL);
         gs_debug_shutdown ();
 
-	return 0;
+        return 0;
 }
