@@ -44,13 +44,20 @@
 #include <libgnomekbd/gkbd-indicator.h>
 #endif
 
+#ifdef WITH_LIBNOTIFY
+#include <libnotify/notify.h>
+#endif
+
 #include "gs-lock-plug.h"
 
 #include "gs-debug.h"
 
 #define KEY_LOCK_DIALOG_THEME "/apps/gnome-screensaver/lock_dialog_theme"
 #define GDM_FLEXISERVER_COMMAND "gdmflexiserver"
-#define GDM_FLEXISERVER_ARGS    "--startnew"
+#define GDM_FLEXISERVER_ARGS    "--startnew Standard"
+
+/* same as SMS ;) */
+#define NOTE_BUFFER_MAX_CHARS 160
 
 enum {
         AUTH_PAGE = 0,
@@ -77,19 +84,27 @@ struct GSLockPlugPrivate
         GtkWidget   *auth_prompt_box;
         GtkWidget   *auth_capslock_label;
         GtkWidget   *auth_message_label;
-        GtkWidget   *switch_user_treeview;
+        GtkWidget   *away_message_label;
 
         GtkWidget   *auth_unlock_button;
         GtkWidget   *auth_switch_button;
         GtkWidget   *auth_cancel_button;
         GtkWidget   *auth_logout_button;
+        GtkWidget   *auth_note_button;
+        GtkWidget   *note_tab;
+        GtkWidget   *note_tab_label;
+        GtkWidget   *note_text_view;
+        GtkWidget   *note_ok_button;
+        GtkWidget   *note_cancel_button;
 
         GtkWidget   *auth_prompt_kbd_layout_indicator;
 
         gboolean     caps_lock_on;
         gboolean     switch_enabled;
+        gboolean     leave_note_enabled;
         gboolean     logout_enabled;
         char        *logout_command;
+        char        *away_message;
 
         guint        timeout;
 
@@ -118,7 +133,8 @@ enum {
         PROP_0,
         PROP_LOGOUT_ENABLED,
         PROP_LOGOUT_COMMAND,
-        PROP_SWITCH_ENABLED
+        PROP_SWITCH_ENABLED,
+        PROP_AWAY_MESSAGE
 };
 
 static guint lock_plug_signals [LAST_SIGNAL];
@@ -603,6 +619,56 @@ is_program_in_path (const char *program)
         }
 }
 
+static void
+gs_lock_plug_set_away_message (GSLockPlug *plug,
+                               const char *away_message)
+{
+        g_return_if_fail (GS_LOCK_PLUG (plug));
+
+        g_free (plug->priv->away_message);
+        plug->priv->away_message = g_strdup (away_message);
+
+        if (plug->priv->away_message_label) {
+                if (plug->priv->away_message) {
+                        gtk_label_set_text (GTK_LABEL (plug->priv->away_message_label),
+                                            plug->priv->away_message);
+                        gtk_widget_show (plug->priv->away_message_label);
+                }
+                else {
+                        gtk_widget_hide (plug->priv->away_message_label);
+                }
+        }
+}
+
+static void
+gs_lock_plug_get_property (GObject    *object,
+                           guint       prop_id,
+                           GValue     *value,
+                           GParamSpec *pspec)
+{
+        GSLockPlug *self;
+
+        self = GS_LOCK_PLUG (object);
+
+        switch (prop_id) {
+        case PROP_LOGOUT_ENABLED:
+                g_value_set_boolean (value, self->priv->logout_enabled);
+                break;
+        case PROP_LOGOUT_COMMAND:
+                g_value_set_string (value, self->priv->logout_command);
+                break;
+        case PROP_SWITCH_ENABLED:
+                g_value_set_boolean (value, self->priv->switch_enabled);
+                break;
+        case PROP_AWAY_MESSAGE:
+                g_value_set_string (value, self->priv->away_message);
+                break;
+        default:
+                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+                break;
+        }
+}
+
 static void
 gs_lock_plug_set_switch_enabled (GSLockPlug *plug,
                                  gboolean    switch_enabled)
@@ -651,6 +717,9 @@ gs_lock_plug_set_property (GObject            *object,
         case PROP_LOGOUT_COMMAND:
                 gs_lock_plug_set_logout_command (self, g_value_get_string (value));
                 break;
+        case PROP_AWAY_MESSAGE:
+                gs_lock_plug_set_away_message (self, g_value_get_string (value));
+                break;
         case PROP_SWITCH_ENABLED:
                 gs_lock_plug_set_switch_enabled (self, g_value_get_boolean (value));
                 break;
@@ -660,32 +729,6 @@ gs_lock_plug_set_property (GObject            *object,
         }
 }
 
-static void
-gs_lock_plug_get_property (GObject    *object,
-                           guint       prop_id,
-                           GValue     *value,
-                           GParamSpec *pspec)
-{
-        GSLockPlug *self;
-
-        self = GS_LOCK_PLUG (object);
-
-        switch (prop_id) {
-        case PROP_LOGOUT_ENABLED:
-                g_value_set_boolean (value, self->priv->logout_enabled);
-                break;
-        case PROP_LOGOUT_COMMAND:
-                g_value_set_string (value, self->priv->logout_command);
-                break;
-        case PROP_SWITCH_ENABLED:
-                g_value_set_boolean (value, self->priv->switch_enabled);
-                break;
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
 static void
 gs_lock_plug_close (GSLockPlug *plug)
 {
@@ -752,6 +795,13 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                                NULL,
                                                                NULL,
                                                                G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_AWAY_MESSAGE,
+                                         g_param_spec_string ("away-message",
+                                                               NULL,
+                                                               NULL,
+                                                               NULL,
+                                                               G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
                                          PROP_SWITCH_ENABLED,
                                          g_param_spec_boolean ("switch-enabled",
@@ -766,6 +816,72 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                       "close", 0);
 }
 
+static void
+take_note (GtkButton  *button,
+           GSLockPlug *plug)
+{
+        int page;
+
+        page = gtk_notebook_page_num (GTK_NOTEBOOK (plug->priv->notebook), plug->priv->note_tab);
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), page);
+        /* this counts as activity so restart the timer */
+        restart_cancel_timeout (plug);
+}
+
+static void
+submit_note (GtkButton  *button,
+             GSLockPlug *plug)
+{
+#ifdef WITH_LIBNOTIFY
+        char               *text;
+        char                summary[128];
+        char               *escaped_text;
+        GtkTextBuffer      *buffer;
+        GtkTextIter         start, end;
+        time_t              t;
+        struct tm          *tmp;
+        NotifyNotification *note;
+
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), AUTH_PAGE);
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (plug->priv->note_text_view));
+        gtk_text_buffer_get_bounds (buffer, &start, &end);
+        text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+        gtk_text_buffer_set_text (buffer, "", 0);
+        escaped_text = g_markup_escape_text (text, -1);
+
+        t = time (NULL);
+        tmp = localtime (&t);
+        strftime (summary, 128, "%X", tmp);
+
+        notify_init ("gnome-screensaver-dialog");
+        note = notify_notification_new (summary, escaped_text, NULL, NULL);
+        notify_notification_set_timeout (note, NOTIFY_EXPIRES_NEVER);
+        notify_notification_show (note, NULL);
+        g_object_unref (note);
+
+        g_free (text);
+        g_free (escaped_text);
+
+        gs_lock_plug_response (plug, GS_LOCK_PLUG_RESPONSE_CANCEL);
+#endif /* WITH_LIBNOTIFY */
+}
+
+static void
+cancel_note (GtkButton  *button,
+             GSLockPlug *plug)
+{
+        GtkTextBuffer *buffer;
+
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), AUTH_PAGE);
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (plug->priv->note_text_view));
+        gtk_text_buffer_set_text (buffer, "", 0);
+
+        /* this counts as activity so restart the timer */
+        restart_cancel_timeout (plug);
+
+        gtk_window_set_default (GTK_WINDOW (plug), plug->priv->auth_unlock_button);
+}
+
 static void
 logout_button_clicked (GtkButton  *button,
                        GSLockPlug *plug)
@@ -915,13 +1031,13 @@ entry_key_press (GtkWidget   *widget,
                 return FALSE;
         }
 
-	if (strcmp (event->string, "") == 0) {
-		return FALSE;
+        if (strcmp (event->string, "") == 0) {
+                return FALSE;
         }
 
-	queue_key_event (plug, event);
+        queue_key_event (plug, event);
 
-	return TRUE;
+        return TRUE;
 }
 
 /* adapted from gtk_dialog_add_button */
@@ -987,10 +1103,10 @@ get_user_name (void)
 
 static gboolean
 check_user_file (const gchar *filename,
-		 uid_t        user,
-		 gssize       max_file_size,
-		 gboolean     relax_group,
-		 gboolean     relax_other)
+                 uid_t        user,
+                 gssize       max_file_size,
+                 gboolean     relax_group,
+                 gboolean     relax_other)
 {
         struct stat fileinfo;
 
@@ -1260,6 +1376,7 @@ create_page_one (GSLockPlug *plug)
         gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_realname_label), TRUE);
         gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_realname_label, FALSE, FALSE, 0);
 
+        /* To translators: This expands to USERNAME on HOSTNAME */
         str = g_strdup_printf ("<span size=\"small\">%s</span>", _("%U on %h"));
         plug->priv->auth_username_label = gtk_label_new (str);
         g_free (str);
@@ -1414,6 +1531,12 @@ load_theme (GSLockPlug *plug)
         plug->priv->auth_cancel_button = glade_xml_get_widget (xml, "auth-cancel-button");
         plug->priv->auth_logout_button = glade_xml_get_widget (xml, "auth-logout-button");
         plug->priv->auth_switch_button = glade_xml_get_widget (xml, "auth-switch-button");
+        plug->priv->auth_note_button = glade_xml_get_widget (xml, "auth-note-button");
+        plug->priv->note_tab = glade_xml_get_widget (xml, "note-tab");
+        plug->priv->note_tab_label = glade_xml_get_widget (xml, "note-tab-label");
+        plug->priv->note_ok_button = glade_xml_get_widget (xml, "note-ok-button");
+        plug->priv->note_text_view = glade_xml_get_widget (xml, "note-text-view");
+        plug->priv->note_cancel_button = glade_xml_get_widget (xml, "note-cancel-button");
 
         /* Placeholder for the keyboard indicator */
         plug->priv->auth_prompt_kbd_layout_indicator = glade_xml_get_widget (xml, "auth-prompt-kbd-layout-indicator");
@@ -1423,9 +1546,14 @@ load_theme (GSLockPlug *plug)
         if (plug->priv->auth_switch_button != NULL) {
                 gtk_widget_set_no_show_all (plug->priv->auth_switch_button, TRUE);
         }
+        if (plug->priv->auth_note_button != NULL) {
+                gtk_widget_set_no_show_all (plug->priv->auth_note_button, TRUE);
+        }
 
         gtk_widget_show_all (lock_dialog);
 
+        plug->priv->away_message_label = glade_xml_get_widget (xml, "away-message-label");
+
         return TRUE;
 }
 
@@ -1439,6 +1567,31 @@ delete_handler (GSLockPlug  *plug,
         return TRUE; /* Do not destroy */
 }
 
+static void
+clear_clipboards (GSLockPlug *plug)
+{
+        GtkClipboard *clipboard;
+
+        clipboard = gtk_widget_get_clipboard (GTK_WIDGET (plug), GDK_SELECTION_PRIMARY);
+        gtk_clipboard_clear (clipboard);
+        gtk_clipboard_set_text (clipboard, "", -1);
+        clipboard = gtk_widget_get_clipboard (GTK_WIDGET (plug), GDK_SELECTION_CLIPBOARD);
+        gtk_clipboard_clear (clipboard);
+        gtk_clipboard_set_text (clipboard, "", -1);
+}
+
+static void
+on_note_text_buffer_changed (GtkTextBuffer *buffer,
+                             GSLockPlug    *plug)
+{
+        int len;
+
+        len = gtk_text_buffer_get_char_count (buffer);
+        if (len > NOTE_BUFFER_MAX_CHARS) {
+                gtk_widget_set_sensitive (plug->priv->note_text_view, FALSE);
+        }
+}
+
 #define INVISIBLE_CHAR_DEFAULT       '*'
 #define INVISIBLE_CHAR_BLACK_CIRCLE  0x25cf
 #define INVISIBLE_CHAR_WHITE_BULLET  0x25e6
@@ -1454,6 +1607,14 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         plug->priv = GS_LOCK_PLUG_GET_PRIVATE (plug);
 
+        clear_clipboards (plug);
+
+#ifdef WITH_LIBNOTIFY
+        plug->priv->leave_note_enabled = TRUE;
+#else
+        plug->priv->leave_note_enabled = FALSE;
+#endif
+
         if (! load_theme (plug)) {
                 gs_debug ("Unable to load theme!");
 
@@ -1475,7 +1636,13 @@ gs_lock_plug_init (GSLockPlug *plug)
                 gtk_widget_show_all (plug->priv->vbox);
         }
 
-	/* Layout indicator */
+        if (plug->priv->note_text_view != NULL) {
+                GtkTextBuffer *buffer;
+                buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (plug->priv->note_text_view));
+                g_signal_connect (buffer, "changed", G_CALLBACK (on_note_text_buffer_changed), plug);
+        }
+
+        /* Layout indicator */
 #ifdef WITH_KBD_LAYOUT_INDICATOR
         if (plug->priv->auth_prompt_kbd_layout_indicator != NULL) {
                 XklEngine *engine;
@@ -1486,9 +1653,11 @@ gs_lock_plug_init (GSLockPlug *plug)
 
                         layout_indicator = gkbd_indicator_new ();
                         gkbd_indicator_set_parent_tooltips (GKBD_INDICATOR (layout_indicator), TRUE);
-                        gtk_misc_set_alignment (GTK_MISC (layout_indicator), 1, 0.5);
                         gtk_box_pack_start (GTK_BOX (plug->priv->auth_prompt_kbd_layout_indicator),
-                                            layout_indicator, FALSE, FALSE, 6);
+                                            layout_indicator,
+                                            FALSE,
+                                            FALSE,
+                                            6);
 
                         gtk_widget_show_all (layout_indicator);
                         gtk_widget_show (plug->priv->auth_prompt_kbd_layout_indicator);
@@ -1497,9 +1666,24 @@ gs_lock_plug_init (GSLockPlug *plug)
                 }
 
                 g_object_unref (engine);
-	}
+        }
 #endif
 
+        if (plug->priv->auth_note_button != NULL) {
+                if (plug->priv->leave_note_enabled) {
+                        gtk_widget_show_all (plug->priv->auth_note_button);
+                } else {
+                        gtk_widget_hide (plug->priv->auth_note_button);
+                }
+        }
+        if (plug->priv->auth_switch_button != NULL) {
+                if (plug->priv->switch_enabled) {
+                        gtk_widget_show_all (plug->priv->auth_switch_button);
+                } else {
+                        gtk_widget_hide (plug->priv->auth_switch_button);
+                }
+        }
+
         gtk_widget_grab_default (plug->priv->auth_unlock_button);
 
         if (plug->priv->auth_username_label != NULL) {
@@ -1539,11 +1723,34 @@ gs_lock_plug_init (GSLockPlug *plug)
         g_signal_connect (plug->priv->auth_cancel_button, "clicked",
                           G_CALLBACK (cancel_button_clicked), plug);
 
+        if (plug->priv->away_message_label) {
+              if (plug->priv->away_message) {
+                      gtk_label_set_text (GTK_LABEL (plug->priv->away_message_label),
+                                          plug->priv->away_message);
+              }
+              else {
+                      gtk_widget_hide (plug->priv->away_message_label);
+              }
+        }
+
         if (plug->priv->auth_switch_button != NULL) {
                 g_signal_connect (plug->priv->auth_switch_button, "clicked",
                                   G_CALLBACK (switch_user_button_clicked), plug);
         }
 
+        if (plug->priv->auth_note_button != NULL) {
+                g_signal_connect (plug->priv->auth_note_button, "clicked",
+                                  G_CALLBACK (take_note), plug);
+                g_signal_connect (plug->priv->note_ok_button, "clicked",
+                                  G_CALLBACK (submit_note), plug);
+                g_signal_connect (plug->priv->note_cancel_button, "clicked",
+                                  G_CALLBACK (cancel_note), plug);
+        }
+
+        if (plug->priv->note_tab_label != NULL) {
+                expand_string_for_label (plug->priv->note_tab_label);
+        }
+
         if (plug->priv->auth_logout_button != NULL) {
                 g_signal_connect (plug->priv->auth_logout_button, "clicked",
                                   G_CALLBACK (logout_button_clicked), plug);
